import { HttpApi } from "aws-cdk-lib/aws-apigatewayv2";
import * as acm from "aws-cdk-lib/aws-certificatemanager";
import * as cloudfront from "aws-cdk-lib/aws-cloudfront";
import * as route53 from "aws-cdk-lib/aws-route53";
import * as wafv2 from "aws-cdk-lib/aws-wafv2";
import { Construct } from "constructs";
/**
 * @interface WafHttpApiProps
 * @description Properties for configuring the WafHttpApi construct.
 * This interface defines all the configuration options available when creating
 * a WAF-protected HTTP API with CloudFront distribution and optional custom domain support.
 */
export interface WafHttpApiProps {
    /**
     * The HTTP API to be protected by the WAF and CloudFront.
     * This should be an instance of `aws-cdk-lib/aws-apigatewayv2.HttpApi`.
     * The API will be fronted by a CloudFront distribution with WAF protection.
     *
     * @type {HttpApi}
     * @example
     * const httpApi = new HttpApi(this, 'MyApi', {
     *   description: 'My protected HTTP API'
     * });
     */
    readonly httpApi: HttpApi;
    /**
     * Optional: Custom WAF rules to apply to the WebACL.
     * If not provided, a default set of AWS Managed Rules will be used,
     * specifically "AWSManagedRulesAmazonIpReputationList" and "AWSManagedRulesCommonRuleSet".
     * These rules help protect against common web exploits and unwanted traffic.
     *
     * @type {wafv2.CfnWebACL.RuleProperty[]}
     * @default AWS Managed Rules (AmazonIpReputationList, CommonRuleSet)
     * @example
     * wafRules: [
     *   {
     *     name: 'RateLimitRule',
     *     priority: 10,
     *     statement: {
     *       rateBasedStatement: {
     *         limit: 2000,
     *         aggregateKeyType: 'IP',
     *       },
     *     },
     *     action: { block: {} },
     *     visibilityConfig: {
     *       cloudWatchMetricsEnabled: true,
     *       metricName: 'RateLimitRule',
     *       sampledRequestsEnabled: true,
     *     },
     *   },
     * ]
     */
    readonly wafRules?: wafv2.CfnWebACL.RuleProperty[];
    /**
     * Optional: Custom domain name for the CloudFront distribution.
     * When provided, the CloudFront distribution will be configured to accept requests on this domain.
     * If no certificate is provided, an ACM certificate will be automatically generated with DNS validation.
     *
     * Supports various domain formats:
     * - Apex domains: `example.com`
     * - Subdomains: `api.example.com`, `www.api.example.com`
     * - Wildcard domains: `*.example.com`
     *
     * @type {string}
     * @example
     * // Apex domain
     * domain: 'example.com'
     *
     * // Subdomain
     * domain: 'api.example.com'
     *
     * // Wildcard domain
     * domain: '*.api.example.com'
     */
    readonly domain?: string;
    /**
     * Optional: SSL certificate for the custom domain.
     * Must be an ACM certificate in the us-east-1 region for CloudFront compatibility.
     * If not provided and a domain is specified, a certificate will be automatically generated
     * using DNS validation.
     *
     * **Important Requirements:**
     * - Certificate must be in us-east-1 region (CloudFront requirement)
     * - Certificate must cover the specified domain (exact match or wildcard)
     * - Certificate must be valid and accessible
     *
     * @type {acm.ICertificate}
     * @example
     * // Using existing certificate
     * const existingCert = Certificate.fromCertificateArn(
     *   this,
     *   'ExistingCert',
     *   'arn:aws:acm:us-east-1:123456789012:certificate/12345678-1234-1234-1234-123456789012'
     * );
     *
     * // In props
     * certificate: existingCert
     */
    readonly certificate?: acm.ICertificate;
    /**
     * Optional: Route 53 hosted zone for automatic DNS record creation.
     * When provided along with a domain, the construct will automatically create
     * Route 53 A and AAAA records pointing to the CloudFront distribution.
     *
     * **Behavior:**
     * - When both `hostedZone` and `domain` are provided: DNS records are automatically created
     * - When `hostedZone` is provided without `domain`: Hosted zone is ignored with warning
     * - When `domain` is provided without `hostedZone`: No DNS records are created
     * - Domain must match or be a subdomain of the hosted zone's domain
     *
     * @type {route53.IHostedZone}
     * @example
     * // Using existing hosted zone
     * const hostedZone = HostedZone.fromLookup(this, 'MyZone', {
     *   domainName: 'example.com'
     * });
     *
     * // In props with automatic DNS record creation
     * const protectedApi = new WafHttpApi(this, 'MyApi', {
     *   httpApi: myHttpApi,
     *   domain: 'api.example.com',
     *   hostedZone: hostedZone
     * });
     *
     * // Access created DNS records
     * if (protectedApi.aRecord) {
     *   new CfnOutput(this, 'ARecordName', {
     *     value: protectedApi.aRecord.domainName
     *   });
     * }
     */
    readonly hostedZone?: route53.IHostedZone;
}
/**
 * @class WafHttpApi
 * @extends Construct
 * @description A CDK construct that fronts an AWS HTTP API with a CloudFront distribution
 * and protects it with AWS WAF. This enhances security and performance by
 * adding a global CDN layer and web application firewall capabilities.
 * It also injects a secret header from CloudFront to the origin to allow
 * for origin verification by a Lambda Authorizer or similar mechanism.
 *
 * **Custom Domain Support:**
 * This construct supports custom domains with automatic SSL certificate management.
 * When a domain is provided, the CloudFront distribution will be configured to accept
 * requests on that domain. If no certificate is provided, an ACM certificate will be
 * automatically generated with DNS validation.
 *
 * @example
 * // Basic usage without custom domain
 * const protectedApi = new WafHttpApi(this, 'MyProtectedApi', {
 *   httpApi: myHttpApi,
 * });
 *
 * @example
 * // Usage with custom domain (automatic certificate generation)
 * const protectedApiWithDomain = new WafHttpApi(this, 'MyProtectedApi', {
 *   httpApi: myHttpApi,
 *   domain: 'api.example.com',
 * });
 *
 * @example
 * // Usage with custom domain and provided certificate
 * const protectedApiWithCert = new WafHttpApi(this, 'MyProtectedApi', {
 *   httpApi: myHttpApi,
 *   domain: 'api.example.com',
 *   certificate: existingCertificate,
 * });
 *
 * @example
 * // Usage with wildcard domain
 * const wildcardApi = new WafHttpApi(this, 'WildcardApi', {
 *   httpApi: myHttpApi,
 *   domain: '*.api.example.com',
 * });
 *
 * @example
 * // Usage with custom WAF rules and domain
 * const customRulesApi = new WafHttpApi(this, 'CustomRulesApi', {
 *   httpApi: myHttpApi,
 *   domain: 'secure-api.example.com',
 *   wafRules: [
 *     {
 *       name: 'RateLimitRule',
 *       priority: 10,
 *       statement: {
 *         rateBasedStatement: {
 *           limit: 2000,
 *           aggregateKeyType: 'IP',
 *         },
 *       },
 *       action: { block: {} },
 *       visibilityConfig: {
 *         cloudWatchMetricsEnabled: true,
 *         metricName: 'RateLimitRule',
 *         sampledRequestsEnabled: true,
 *       },
 *     },
 *   ],
 * });
 *
 * @example
 * // Usage with hosted zone for automatic DNS record creation
 * const hostedZone = HostedZone.fromLookup(this, 'MyZone', {
 *   domainName: 'example.com'
 * });
 *
 * const apiWithDns = new WafHttpApi(this, 'ApiWithDNS', {
 *   httpApi: myHttpApi,
 *   domain: 'api.example.com',
 *   hostedZone: hostedZone
 * });
 *
 * // Access the automatically created DNS records
 * if (apiWithDns.aRecord) {
 *   new CfnOutput(this, 'ARecordName', {
 *     value: apiWithDns.aRecord.domainName,
 *     description: 'A record for the API domain'
 *   });
 * }
 *
 * if (apiWithDns.aaaaRecord) {
 *   new CfnOutput(this, 'AAAARecordName', {
 *     value: apiWithDns.aaaaRecord.domainName,
 *     description: 'AAAA record for the API domain'
 *   });
 * }
 */
export declare class WafHttpApi extends Construct {
    /**
     * @static
     * @readonly
     * @property {string} SECRET_HEADER_NAME
     * @description The name of the custom header CloudFront will add to requests
     * forwarded to the origin. This header can be used by your backend (e.g.,
     * a Lambda Authorizer for API Gateway) to verify that the request originated
     * from CloudFront and not directly from the internet.
     */
    static readonly SECRET_HEADER_NAME = "X-Origin-Verify";
    /**
     * The CloudFront distribution created and managed by this construct.
     * You can use this property to retrieve the distribution's domain name or ARN.
     *
     * @readonly
     * @type {cloudfront.Distribution}
     * @example
     * // Access the CloudFront distribution domain name
     * const distributionDomain = wafHttpApi.distribution.distributionDomainName;
     *
     * // Access the distribution ARN
     * const distributionArn = wafHttpApi.distribution.distributionArn;
     *
     * // Use in CloudFormation outputs
     * new CfnOutput(this, 'DistributionEndpoint', {
     *   value: `https://${wafHttpApi.distribution.distributionDomainName}`,
     *   description: 'CloudFront distribution endpoint'
     * });
     */
    readonly distribution: cloudfront.Distribution;
    /**
     * The randomly generated secret value for the custom header.
     * This value is unique for each deployment of the construct and should be used
     * in your HTTP API's authorizer or backend logic to validate that requests
     * are coming through CloudFront and not directly from the internet.
     *
     * @readonly
     * @type {string}
     * @example
     * // Use in Lambda authorizer
     * export const handler = async (event: APIGatewayProxyEvent) => {
     *   const secretHeader = event.headers[WafHttpApi.SECRET_HEADER_NAME];
     *   const expectedSecret = process.env.CLOUDFRONT_SECRET; // Set from wafHttpApi.secretHeaderValue
     *
     *   if (secretHeader !== expectedSecret) {
     *     throw new Error('Unauthorized: Request not from CloudFront');
     *   }
     *
     *   // Continue with request processing...
     * };
     *
     * // Set as environment variable in Lambda
     * const lambda = new NodejsFunction(this, 'ApiHandler', {
     *   environment: {
     *     CLOUDFRONT_SECRET: wafHttpApi.secretHeaderValue
     *   }
     * });
     */
    readonly secretHeaderValue: string;
    /**
     * The SSL certificate used for the custom domain.
     * This property will be defined in the following scenarios:
     * - When a certificate is provided via the `certificate` prop
     * - When a certificate is automatically generated for a custom domain
     *
     * The property will be `undefined` when no custom domain is configured.
     *
     * @readonly
     * @type {acm.ICertificate | undefined}
     * @example
     * // Check if certificate is available
     * if (wafHttpApi.certificate) {
     *   // Output certificate ARN
     *   new CfnOutput(this, 'CertificateArn', {
     *     value: wafHttpApi.certificate.certificateArn,
     *     description: 'SSL certificate ARN'
     *   });
     *
     *   // Use certificate in other resources
     *   const loadBalancer = new ApplicationLoadBalancer(this, 'ALB', {
     *     // ... other props
     *   });
     *
     *   loadBalancer.addListener('HttpsListener', {
     *     port: 443,
     *     certificates: [wafHttpApi.certificate],
     *     // ... other listener props
     *   });
     * }
     */
    readonly certificate?: acm.ICertificate;
    /**
     * The custom domain name configured for this distribution.
     * This property will be defined when a domain is provided via the `domain` prop.
     * It will be `undefined` when no custom domain is configured.
     *
     * @readonly
     * @type {string | undefined}
     * @example
     * // Check if custom domain is configured
     * if (wafHttpApi.customDomain) {
     *   // Output custom domain endpoint
     *   new CfnOutput(this, 'CustomDomainEndpoint', {
     *     value: `https://${wafHttpApi.customDomain}`,
     *     description: 'Custom domain API endpoint'
     *   });
     *
     *   // Use domain in Route53 record
     *   new ARecord(this, 'ApiRecord', {
     *     zone: hostedZone,
     *     recordName: wafHttpApi.customDomain,
     *     target: RecordTarget.fromAlias(
     *       new CloudFrontTarget(wafHttpApi.distribution)
     *     ),
     *   });
     * } else {
     *   // Use CloudFront default domain
     *   new CfnOutput(this, 'DefaultEndpoint', {
     *     value: `https://${wafHttpApi.distribution.distributionDomainName}`,
     *     description: 'Default CloudFront endpoint'
     *   });
     * }
     */
    readonly customDomain?: string;
    /**
     * The Route 53 A record created for the custom domain.
     * This property will be defined when both `hostedZone` and `domain` are provided,
     * and the construct automatically creates DNS records pointing to the CloudFront distribution.
     *
     * The A record maps the custom domain to the CloudFront distribution's IPv4 addresses.
     *
     * @readonly
     * @type {route53.ARecord | undefined}
     * @example
     * // Check if A record was created
     * if (wafHttpApi.aRecord) {
     *   // Output A record details
     *   new CfnOutput(this, 'ARecordName', {
     *     value: wafHttpApi.aRecord.domainName,
     *     description: 'A record domain name'
     *   });
     *
     *   // Reference the record in other resources
     *   const recordArn = wafHttpApi.aRecord.recordArn;
     * }
     *
     * // The A record will be undefined if:
     * // - No hostedZone was provided
     * // - No domain was provided
     * // - hostedZone was provided without domain (ignored with warning)
     */
    readonly aRecord?: route53.ARecord;
    /**
     * The Route 53 AAAA record created for the custom domain.
     * This property will be defined when both `hostedZone` and `domain` are provided,
     * and the construct automatically creates DNS records pointing to the CloudFront distribution.
     *
     * The AAAA record maps the custom domain to the CloudFront distribution's IPv6 addresses.
     *
     * @readonly
     * @type {route53.AaaaRecord | undefined}
     * @example
     * // Check if AAAA record was created
     * if (wafHttpApi.aaaaRecord) {
     *   // Output AAAA record details
     *   new CfnOutput(this, 'AAAARecordName', {
     *     value: wafHttpApi.aaaaRecord.domainName,
     *     description: 'AAAA record domain name'
     *   });
     *
     *   // Reference the record in other resources
     *   const recordArn = wafHttpApi.aaaaRecord.recordArn;
     * }
     *
     * // The AAAA record will be undefined if:
     * // - No hostedZone was provided
     * // - No domain was provided
     * // - hostedZone was provided without domain (ignored with warning)
     */
    readonly aaaaRecord?: route53.AaaaRecord;
    /**
     * @constructor
     * @param {Construct} scope The scope in which to define this construct (e.g., a CDK Stack).
     * @param {string} id The unique identifier for this construct within its scope.
     * @param {WafHttpApiProps} props The properties required to configure this construct,
     * including the target HTTP API, optional WAF rules, custom domain, and SSL certificate.
     *
     * **Props Configuration:**
     * - `httpApi` (required): The HTTP API Gateway to protect
     * - `wafRules` (optional): Custom WAF rules, defaults to AWS managed rules
     * - `domain` (optional): Custom domain name for the CloudFront distribution
     * - `certificate` (optional): SSL certificate for the custom domain (must be in us-east-1)
     *
     * **Custom Domain Behavior:**
     * - If `domain` is provided without `certificate`: ACM certificate is auto-generated
     * - If both `domain` and `certificate` are provided: Uses the provided certificate
     * - If `certificate` is provided without `domain`: Certificate is ignored with warning
     * - If neither is provided: Uses default CloudFront domain only
     */
    constructor(scope: Construct, id: string, props: WafHttpApiProps);
    /**
     * @private
     * @method validateDomainFormat
     * @description Validates the format of a domain name using regex pattern matching.
     * Supports apex domains (example.com), subdomains (api.example.com), and wildcard domains (*.example.com).
     * @param {string} domain The domain name to validate
     * @throws {Error} Throws an error with descriptive message if domain format is invalid
     */
    private validateDomainFormat;
    /**
     * @private
     * @method validateCertificateRegion
     * @description Validates that the provided certificate is in the us-east-1 region.
     * CloudFront requires certificates to be in us-east-1 region for custom domains.
     * @param {acm.ICertificate} certificate The certificate to validate
     * @throws {Error} Throws an error if certificate is not in us-east-1 region
     */
    private validateCertificateRegion;
    /**
     * @private
     * @method validateCertificateDomainCompatibility
     * @description Validates that the provided certificate covers the specified domain.
     * This ensures the certificate can be used for the custom domain configuration.
     * @param {acm.ICertificate} certificate The certificate to validate
     * @param {string} domain The domain that should be covered by the certificate
     * @throws {Error} Throws an error if certificate doesn't cover the domain
     */
    private validateCertificateDomainCompatibility;
    /**
     * @private
     * @method validateHostedZoneDomainCompatibility
     * @description Validates that the provided domain matches or is a subdomain of the hosted zone's domain.
     * This ensures that DNS records can be properly created in the hosted zone.
     * @param {route53.IHostedZone} hostedZone The hosted zone to validate against
     * @param {string} domain The domain that should belong to the hosted zone
     * @throws {Error} Throws an error if domain doesn't match or isn't a subdomain of the hosted zone
     */
    private validateHostedZoneDomainCompatibility;
    /**
     * @private
     * @method createDefaultRules
     * @description Creates a default set of AWS Managed Rules for the WAF WebACL.
     * These rules provide a good baseline of protection against common threats.
     * @returns {wafv2.CfnWebACL.RuleProperty[]} An array of WAF rule properties.
     */
    private createDefaultRules;
}
